@@ -149,6 +149,15 @@ static inline void *usbi_reallocf(void *ptr, size_t size)
 
 #define TIMESPEC_IS_SET(ts) ((ts)->tv_sec != 0 || (ts)->tv_nsec != 0)
 
+/* Some platforms don't have this define */
+#ifndef TIMESPEC_TO_TIMEVAL
+#define TIMESPEC_TO_TIMEVAL(tv, ts)                                     \
+        do {                                                            \
+                (tv)->tv_sec = (ts)->tv_sec;                            \
+                (tv)->tv_usec = (ts)->tv_nsec / 1000;                   \
+        } while (0)
+#endif
+
 void usbi_log(struct libusb_context *ctx, enum libusb_log_level level,
 	const char *function, const char *format, ...);
 
@@ -534,8 +543,8 @@ struct usbi_os_backend {
 	 *
 	 * After computing a session ID for a device, call
 	 * usbi_get_device_by_session_id(). This function checks if libusbx already
-	 * knows about the device, and if so, it provides you with a libusb_device
-	 * structure for it.
+	 * knows about the device, and if so, it provides you with a reference
+	 * to a libusb_device structure for it.
 	 *
 	 * If usbi_get_device_by_session_id() returns NULL, it is time to allocate
 	 * a new device structure for the device. Call usbi_alloc_device() to
