@@ -49,7 +49,7 @@
  * sysfs allows us to read the kernel's in-memory copies of device descriptors
  * and so forth, avoiding the need to open the device:
  *  - The binary "descriptors" file contains all config descriptors since
- *    2.6.26, commit 217a9081d8e69026186067711131b77f0ce219ed 
+ *    2.6.26, commit 217a9081d8e69026186067711131b77f0ce219ed
  *  - The binary "descriptors" file was added in 2.6.23, commit
  *    69d42a78f935d19384d1f6e4f94b65bb162b36df, but it only contains the
  *    active config descriptors
@@ -564,7 +564,7 @@ static int op_get_device_descriptor(struct libusb_device *dev,
 static int sysfs_get_active_config(struct libusb_device *dev, int *config)
 {
 	char *endptr;
-	char tmp[4] = {0, 0, 0, 0};
+	char tmp[5] = {0, 0, 0, 0, 0};
 	long num;
 	int fd;
 	ssize_t r;
@@ -576,7 +576,7 @@ static int sysfs_get_active_config(struct libusb_device *dev, int *config)
 	r = read(fd, tmp, sizeof(tmp));
 	close(fd);
 	if (r < 0) {
-		usbi_err(DEVICE_CTX(dev), 
+		usbi_err(DEVICE_CTX(dev),
 			"read bConfigurationValue failed ret=%d errno=%d", r, errno);
 		return LIBUSB_ERROR_IO;
 	} else if (r == 0) {
@@ -913,7 +913,7 @@ static int initialize_device(struct libusb_device *dev, uint8_t busnum,
 		}
 		priv->descriptors_len += r;
 	} while (priv->descriptors_len == descriptors_size);
-	
+
 	close(fd);
 
 	if (priv->descriptors_len < DEVICE_DESC_LENGTH) {
@@ -1048,9 +1048,11 @@ int linux_enumerate_device(struct libusb_context *ctx,
 	usbi_dbg("busnum %d devaddr %d session_id %ld", busnum, devaddr,
 		session_id);
 
-	if (usbi_get_device_by_session_id(ctx, session_id)) {
+	dev = usbi_get_device_by_session_id(ctx, session_id);
+	if (dev) {
 		/* device already exists in the context */
 		usbi_dbg("session_id %ld already exists", session_id);
+		libusb_unref_device(dev);
 		return LIBUSB_SUCCESS;
 	}
 
@@ -1101,6 +1103,7 @@ void linux_device_disconnected(uint8_t busnum, uint8_t devaddr, const char *sys_
 		dev = usbi_get_device_by_session_id (ctx, session_id);
 		if (NULL != dev) {
 			usbi_disconnect_device (dev);
+			libusb_unref_device(dev);
 		} else {
 			usbi_dbg("device not found for session %x", session_id);
 		}
@@ -1805,7 +1808,7 @@ static int submit_bulk_transfer(struct usbi_transfer *itransfer,
 					"submiturb failed error %d errno=%d", r, errno);
 				r = LIBUSB_ERROR_IO;
 			}
-	
+
 			/* if the first URB submission fails, we can simply free up and
 			 * return failure immediately. */
 			if (i == 0) {
