@@ -161,9 +161,7 @@ netbsd_get_device_list(struct libusb_context * ctx,
 		session_id = (di.udi_bus << 8 | di.udi_addr);
 		dev = usbi_get_device_by_session_id(ctx, session_id);
 
-		if (dev) {
-			dev = libusb_ref_device(dev);
-		} else {
+		if (dev == NULL) {
 			dev = usbi_alloc_device(ctx, session_id);
 			if (dev == NULL)
 				return (LIBUSB_ERROR_NO_MEM);
